import { ControlUnit } from "./Accessories/ControlUnit";
import { IActivity } from "./Models/Activity";
import { IDeviceSetupItem } from "./Models/DeviceSetupItem";
import { IInput, IOutput, IMatrix } from "./Models/Matrix";
import HarmonyDataProvider from "./DataProviders/HarmonyDataProvider";
import { IConfig, IControlUnit } from "./Models/IConfig";

let Accessory: any;
let Homebridge: any;

/**
 * Main entry.
 * @param homebridge 
 */
export default function (homebridge: any) {
    Homebridge = homebridge;
    Accessory = homebridge.platformAccessory;
    homebridge.registerPlatform(
        'homebridge-harmony-watson',
        'HarmonyHubMatrix',
        HarmonyMatrixPlatform,
        true
    );
};

class HarmonyMatrixPlatform {
    log: any = {};
    config: IConfig;
    api: any;
    dataProvider: HarmonyDataProvider | null;
    controlUnits: Array<ControlUnit> = [];

    constructor(log: any, config: any, api: any) {
        this.log = log;
        this.config = config;
        this.api = api;
        this.log('INFO - Registering Harmony Matrix Platform');
        this.api.on('didFinishLaunching', this.didFinishLaunching.bind(this));

        this.dataProvider = null;
    }

    /**
     * Handler for didFinishLaunching
     * Happens after constructor
     */
    didFinishLaunching() {
        this.log(`Publishing external accessories`);

        //This is required in order to have multiple tv remotes on one platform
        this.controlUnits.forEach((accessory: ControlUnit) => {
            this.api.publishExternalAccessories("HarmonyMatrixPlatform", [accessory.platformAccessory]);
        })
    }

    /**
     * Called by homebridge to gather accessories.
     * @param callback 
     */
    accessories(callback: (accessories: Array<ControlUnit>) => void) {
        //construct data provider
        this.dataProvider = new HarmonyDataProvider({
            hubAddress: this.config.hubIp,
            matrix: this.config.Matrix,
            log: this.log
        });

        this.config.ControlUnits.forEach((unit: IControlUnit) => {
            this.controlUnits.push(new ControlUnit({
                dataProvider: this.dataProvider!,
                displayName: unit.DisplayName,
                api: this.api,
                log: this.log,
                activities: unit.Activities,
                homebridge: Homebridge,
            }));
        });
        callback(this.controlUnits);
    }
}